@@ -2,7 +2,10 @@
 #
 # T2h_i18n.pm: Internationalization for texi2html
 #
-#    Copyright (C) 1999, 2000  Free Software Foundation, Inc.
+#    Copyright (C) 1999-2005  Patrice Dumas <dumas@centre-cired.fr>,
+#                             Derek Price <derek@ximbiot.com>,
+#                             Adrian Aichner <adrian@xemacs.org>,
+#                           & others.
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -16,7 +19,7 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 #
 #-##############################################################################
 
@@ -27,9 +30,6 @@ package Texi2HTML::I18n;
 
 use strict;
 
-use File::Copy;
-use Data::Dumper;
-
 use vars qw(
 @ISA
 @EXPORT
@@ -51,6 +51,9 @@ my $i18n_dir = 'i18n'; # name of the directory containing the per language files
 # To redefine one word, simply do:
 # $T2h_i18n::T2H_LANGUAGES->{<language>}->{<word>} = 'whatever' in your personal init file.
 #
+
+# Those hashes are obsolete but retained here for reference
+
 my $T2H_WORDS_EN =
 {
  # titles  of pages
@@ -327,49 +330,42 @@ sub get_string($;$$)
         }
     }
     return main::substitute_line($string, $state) unless (defined($arguments) or !keys(%$arguments));
+    # if there are arguments, we must protect the %{arg} constructs before
+    # doing substitute_line. So there is a first pass here to change %{arg} 
+    # to %@{arg@}
     my $result = '';
-    while ($string)
-    {
-        if ($string =~ s/^([^%]*)%//)
-        {
-            $result .= $1 if (defined($1));
-            $result .= '%';
-            if ($string =~ s/^%//)
-            {
-                 $result .= '%';
-            }
-            elsif ($string =~ /^\{(\w+)\}/ and exists($arguments->{$1}))
-            {
-                 $string =~ s/^\{(\w+)\}//;
-                 if (!$state->{'keep_texi'})
-                 {
-                      $result .= "\@\{$1\@\}";
-                 }
-                 else
-                 {
-                      $result .= "\{$1\}";
-                 }
-            }
-            else
-            {
-                 $result .= '%';
-            }
-            next;
-        }
-        else 
-        {   
-            $result .= $string;
-            last;
-        }
-    }
     if (!$state->{'keep_texi'})
     {
-         $string = main::substitute_line($result, $state);
-    }
-    else
-    {
-         $string = $result;
+        while ($string)
+        {
+            if ($string =~ s/^([^%]*)%//)
+            {
+                $result .= $1 if (defined($1));
+                $result .= '%';
+                if ($string =~ s/^%//)
+                {
+                     $result .= '%';
+                }
+                elsif ($string =~ /^\{(\w+)\}/ and exists($arguments->{$1}))
+                {
+                     $string =~ s/^\{(\w+)\}//;
+                     $result .= "\@\{$1\@\}";
+                }
+                else
+                {
+                     $result .= '%';
+                }
+                next;
+            }
+            else 
+            {   
+                $result .= $string;
+                last;
+            }
+        }
+        $string = main::substitute_line($result, $state);
     }
+    # now we substitute the arguments 
     $result = '';
     while ($string)
     {
